/**********************************************************************\
**
**                         Copyright (c) 2007-2021 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo_Pmp_PrinterInfo.cpp
**  Facility:    S684015 - Hengstler Extendo Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Exo-PrintMonPlugin)
**  Version:     R9-V2.05-FINAL / 2021SEP14
**  Environment: Microsoft Windows XP/7/8/10 
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Module that manages the printer info structure entry in the registry
**  Note:        -
**  History:     R1-V1.00 / 2007DEC21 / HOR / 1ST RELEASE 
**               -----------------------------------------------------
**               R2-V1.01 / 2008MAY31 / HOR / 2ND RELEASE 
**               -----------------------------------------------------
**               R3-V1.02 / 2010APR30 / HOR / 3RD RELEASE
**               -----------------------------------------------------
**               R4-V2.00a / 2011MAY26 / HOR / M-R4-004-HOR
**                  Complete printer package ported to VS2010, reviewed and adapted Windows 7 32+64 bit
**               R4-V2.00 / 2013JUL18 / HOR / 4TH RELEASE
**               -----------------------------------------------------
**               R5-V2.01 / 2015MAY07 / HOR / 5TH RELEASE
**               -----------------------------------------------------
**               R6-V2.02b / 2016MAY25 / HOR / M-R6-007-HOR
**                  Driver package ported to VS2015 and adapted to Windows 10.
**               R6-V2.02 / 2016MAY25 / HOR / 6TH RELEASE
**               -----------------------------------------------------
**               R7-V2.03 / 2017APR20 / HOR / 7TH RELEASE
**               -----------------------------------------------------
**               R8-V2.04 / 2018JUL02 / HOR / 8TH RELEASE
**               -----------------------------------------------------
**               R9-V2.05 / 2021SEP14 / HOR / 9TH RELEASE
**               -----------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Exo_Pmp_System.h"
#include "Exo_PrintMonPlugin.h"
#include "Exo_Pmp_Interface.h"
#include "Exo_Pmp_RegistryAccess.h"
#include "Exo_Pmp_PrinterInfo.h"

/*
** COMPILER 
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

//******************************************************************************************
// FUNCTION   Exo_Pmp_Pi_InitializePrinterInfo
// ABSTRACT   Initializes printer information 
//******************************************************************************************

bool Exo_Pmp_Pi_InitializePrinterInfo
   (P_EXO_PLUGIN_DATA psData)
{
   bool                       fResult        = false;   // Return value, FALSE by default
   TS_EXO_PMP_PRINTER_INFO    sPrinterInfo;             // Printer info data structure (registry)
   
   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize printer inquiry data structure 
   memset(&sPrinterInfo, 0, sizeof(sPrinterInfo));         // Clear complete structure
   sPrinterInfo.dwSize          = sizeof(sPrinterInfo);    // Set size of this structure
   sPrinterInfo.ubRevision      = D_EXO_PMP_PI_REVISION;   // Set revision number (D_EXO_PMP_PI_REVISION)
   sPrinterInfo.dwTsLastUpdate  = 0;                       // Set timestamp of last update
   sPrinterInfo.dwDataValidFlag = 0;                       // Reset data valid flags (see D_EXO_PMP_PI_DVF_...)

   //-- Set time stamp
   sPrinterInfo.dwTsLastUpdate = GetTickCount();

   //-- Write printer info to registry
   if (!Exo_Pmp_Ra_WritePrinterDataToRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_PRINTER_INFO, (PBYTE) &sPrinterInfo, sizeof(sPrinterInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Exo_Pmp_Pi_UpdatePrinterInfo
// ABSTRACT   Updates the printer info 
//******************************************************************************************

bool Exo_Pmp_Pi_UpdatePrinterInfo
   (P_EXO_PLUGIN_DATA psData, bool fPrinterMightBeBusy, bool fInvalidateOnly)
{
   bool                            fResult                    = false;                  // Return value, FALSE by default
   TS_EXO_PMP_PRINTER_INFO         sPrinterInfo;                                        // Printer info data structure (registry)
   BYTE                            aubEcPrinterInfo[]         = {0x1d,0x61,0x02};       // GS a+[n],n=0x02  => emulation command for requesting printer info
   BYTE                            aubPrinterInfo[sizeof(TS_EXO_API_PCKT_HEADER) + sizeof(TS_EXO_API_PCKT_PRINTER_INFO)]; 
                                                                                        // Printer info bytes
   DWORD                           dwReadTimeOutMs;                                     // Read timeout in ms
   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize printer info structure 
   memset(&sPrinterInfo, 0, sizeof(sPrinterInfo));        // Clear complete structure
   sPrinterInfo.dwSize          = sizeof(sPrinterInfo);   // Set size of this structure
   sPrinterInfo.ubRevision      = D_EXO_PMP_PI_REVISION;  // Set revision number (D_EXO_PMP_PI_REVISION)
   sPrinterInfo.dwTsLastUpdate  = 0;                      // Set timestamp of last update
   sPrinterInfo.dwDataValidFlag = 0;                      // Reset data valid flags (see D_EXO_PMP_PI_DVF_...)

   //-- Check if data must be requested from the printer or if data is to be invalidated
   if (fInvalidateOnly) 
      goto DONE;

   //-- Inquire printer info 
   {
      //-- Inquire printer info 
      //.. Set read timeout printer response to inquiry to .. ms, depending if printer could be busy) 
      dwReadTimeOutMs = (!fPrinterMightBeBusy ? 300 : 15000);
      if (!Exo_Pmp_Ifc_InquirePrinterData(psData, 3, aubEcPrinterInfo, sizeof(aubEcPrinterInfo), aubPrinterInfo, sizeof(aubPrinterInfo), dwReadTimeOutMs)) 
         goto DONE;

      //-- Copy printer info to storage
      memcpy(&sPrinterInfo.sPrinterInfo, &aubPrinterInfo[sizeof(TS_EXO_API_PCKT_HEADER)], sizeof(sPrinterInfo.sPrinterInfo));

      //-- Set printer info valid flag
      sPrinterInfo.dwDataValidFlag |= D_EXO_PMP_PI_DVF_PRINTER_INFO;
   }

      //-- Success (inquiry succesfull)
      fResult = true;

   DONE:

   //-- Set time stamp
   sPrinterInfo.dwTsLastUpdate = GetTickCount();

   //-- Write printer info to registry
   if (!Exo_Pmp_Ra_WritePrinterDataToRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_PRINTER_INFO, (PBYTE) &sPrinterInfo, sizeof(sPrinterInfo)))
      goto FAILURE;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}
