/**********************************************************************\
**
**                         Copyright (c) 2007-2021 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo_Pmp_RegistryAccess.cpp
**  Facility:    S684015 - Hengstler Extendo Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Exo-PrintMonPlugin)
**  Version:     R9-V2.05-FINAL / 2021SEP14
**  Environment: Microsoft Windows XP/7/8/10 
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Functions for registry access
**  Note:        -
**  History:     R1-V1.00 / 2007DEC21 / HOR / 1ST RELEASE 
**               -----------------------------------------------------
**               R2-V1.01 / 2008MAY31 / HOR / 2ND RELEASE 
**               -----------------------------------------------------
**               R3-V1.02 / 2010APR30 / HOR / 3RD RELEASE
**               -----------------------------------------------------
**               R4-V2.00a / 2011MAY26 / HOR / M-R4-004-HOR
**                  Complete printer package ported to VS2010, reviewed and adapted Windows 7 32+64 bit
**               R4-V2.00 / 2013JUL18 / HOR / 4TH RELEASE
**               -----------------------------------------------------
**               R5-V2.01 / 2015MAY07 / HOR / 5TH RELEASE
**               -----------------------------------------------------
**               R6-V2.02b / 2016MAY25 / HOR / M-R6-007-HOR
**                  Driver package ported to VS2015 and adapted to Windows 10.
**               R6-V2.02 / 2016MAY25 / HOR / 6TH RELEASE
**               -----------------------------------------------------
**               R7-V2.03 / 2017APR20 / HOR / 7TH RELEASE
**               -----------------------------------------------------
**               R8-V2.04 / 2018JUL02 / HOR / 8TH RELEASE
**               -----------------------------------------------------
**               R9-V2.05 / 2021SEP14 / HOR / 9TH RELEASE
**               -----------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Exo_Pmp_System.h"
#include "Exo_PrintMonPlugin.h"
#include "Exo_Pmp_Interface.h"
#include "Exo_Pmp_RegistryAccess.h"

/*
** COMPILER
*/

/*
** MACROS 
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

//******************************************************************************************
//** FUNCTION     Exo_Pmp_Ra_WritePrinterDataToRegistry
//** ABSTRACT     Writes data to registry for the current printer
//** PARAMETER    psData                Print monitor plugin data structure 
//**              pszValue              Key value in registry 
//**              pubInfo               Info structure which must be written
//**              dwSize                Size of info structure
//** RETURN       fResult               TRUE if success, FALSE otherwise
//******************************************************************************************

bool Exo_Pmp_Ra_WritePrinterDataToRegistry
   (P_EXO_PLUGIN_DATA psData, PSTR pszValue, PBYTE pubInfo, DWORD dwSize)
{
   bool                            fResult            = false;   // Return value, FALSE by default
   PRINTER_DEFAULTS                sPrinterDefaults;             // Printer defaults structure
   HANDLE                          hPrinter           = NULL;    // Handle of printer
 
   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID) || !pubInfo)
      goto FAILURE;

   //-- Setup printer defaults
   ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
   sPrinterDefaults.DesiredAccess = PRINTER_ALL_ACCESS;
  
   //-- Open printer
   //.. If defined, select printer that is coupled to the document
   //.. otherwise select printer for which the print monitor has been started 
   //.. Note that multiple printers may share the same port
   if (!OpenPrinter(psData->pszPrinterNameDoc ? psData->pszPrinterNameDoc : psData->pszPrinterName, &hPrinter, &sPrinterDefaults)) 
      goto FAILURE;

   //-- Write printer data to registry
   SetPrinterDataExA(hPrinter, D_EXO_PMP_REG_KEY, pszValue, REG_BINARY, pubInfo, dwSize);

   //-- Close printer
   ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Close printer if open
   if (hPrinter) ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
//** FUNCTION     Exo_Pmp_Ra_ReadPrinterDataFromRegistry
//** ABSTRACT     Reads data from registry for the current printer
//** PARAMETER    psData                Print monitor plugin data structure 
//**              pszValue              Key value in registry 
//**              pubInfo               Info structure which must be written
//**              dwSize                Size of info structure
//** RETURN       fResult               TRUE if success, FALSE otherwise
//******************************************************************************************

bool Exo_Pmp_Ra_ReadPrinterDataFromRegistry
   (P_EXO_PLUGIN_DATA psData, PSTR pszValue, PBYTE pubInfo, DWORD dwSize)
{
   bool                            fResult            = false;   // Return value, FALSE by default
   PRINTER_DEFAULTS                sPrinterDefaults;             // Printer defaults structure
   HANDLE                          hPrinter           = NULL;    // Handle of printer
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID) || !pubInfo)
      goto FAILURE;

   //-- Setup printer defaults
   ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
   sPrinterDefaults.DesiredAccess = PRINTER_ALL_ACCESS;
  
   //-- Open printer 
   if (!OpenPrinter(psData->pszPrinterName, &hPrinter, &sPrinterDefaults)) 
      goto FAILURE;

   //-- Read data from registery and reset data if not (yet) available or if revision nr differs
   {
      DWORD dwType;   // Type of registry data
      DWORD dwNeeded; // Bytes needed
   
       if (    (GetPrinterDataExA(hPrinter, D_EXO_PMP_REG_KEY, pszValue, &dwType, pubInfo, dwSize, &dwNeeded) != ERROR_SUCCESS) 
            || (dwType != REG_BINARY)
          ) goto FAILURE;
   }

   //-- Close printer
   ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Close printer if open
   if (hPrinter) ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}
